/*
 * html.c
 *
 * HTML parsing functions for the Quoteriser. These support only a subset
 * of full HTML; refer to the documentation for a full description of the
 * tags available.
 *
 *       Created: 4th March, 1997
 * Last modified: 9th March, 1997
 *
 * (C) 1997 Nicholas Paul Sheppard
 *
 * This file is distributed under the GNU General Public License. See the
 * file COPYING for details.
 */

#include <stdlib.h>
#include <string.h>
#include "html.h"


int HTMLGetNextChunk(char *pszInput, char *pszChunk, char **ppszNext)
/*
 * Get the next portion of the input HTML that can be acted upon as a
 * whole. Chunks can be a whole word, part of a word (i.e. a word
 * interrupted by an HTML tag) or an HTML tag.
 *
 * char *pszInput	- the input HTML
 * char *pszChunk	- the next chunk (output)
 * char *pszNext	- the continuation point (output)
 *
 * Return: HTML_WORD_END  - pszChunk points to a whole word
 *         HTML_WORD_MID  - pszChunk points to part of a word
 *         HTML_TAG_END   - pszChunk points to an HTML tag at the end of a word
 *         HTML_TAG_MID   - pszChunk points to an HTML in the middle of a word
 *         HTML_MACRO_END - pszChunk points to an HTML macro at the end of a word
 *         HTML_MACRO_MID - pszChunk points to an HTML macro in the middle of a word
 *         HTML_END       - we have reached the end of the input
 */
{
	char *pchNextSpace, *pchNextTag, *pchNextMacro, *pchNextEOL;
	char *pszStartChunk;
	int iLength, iRet;

	/* ignore spaces and EOLs */
	pszStartChunk = pszInput;
	while ((pszStartChunk[0] != '\0') && ((pszStartChunk[0] == ' ') || (pszStartChunk[0] == '\n')))
		pszStartChunk++;

	switch (pszStartChunk[0]) {
		case '<':
			/* we at the start of an HTML tag; find the closing bracket, or an EOL */
			pchNextTag = strchr(pszStartChunk, '>');
			pchNextEOL = strchr(pszStartChunk, '\n');

			/* decide which terminator came first */
			(*ppszNext) = pchNextTag;
			if (((*ppszNext) == NULL) || ((pchNextEOL != NULL) && ((*ppszNext) > pchNextEOL)))
				(*ppszNext) = pchNextEOL;
			if ((*ppszNext) == NULL)
				/* this tag is terminated by the end of the input (oops!) */
				(*ppszNext) = strchr(pszInput, '\0');

			/* copy the tag into pszChunk and append closing angle bracket */
			iLength = (*ppszNext) - pszStartChunk;
			strncpy(pszChunk, pszStartChunk, iLength);
			pszChunk[iLength] = '\0';
			strcat(pszChunk, ">");

			/* skip over the angle bracket if we did not insert it */
			if (*(*ppszNext) == '>')
				(*ppszNext) += 1;

			/* determine return value */
			if (((*ppszNext)[0] == ' ') || ((*ppszNext)[0] == '\n') || ((*ppszNext)[0] == '\0'))
				iRet = HTML_TAG_END;
			else
				iRet = HTML_TAG_MID;
			break;

		case '&':
			/* we are at the start of an HTML macro; find the closing semi-colon, or an EOL */
			pchNextMacro = strchr(pszStartChunk, ';');
			pchNextEOL   = strchr(pszStartChunk, '\n');

			/* decide which terminator came first */
			(*ppszNext) = pchNextMacro;
			if (((*ppszNext) == NULL) || ((pchNextEOL != NULL) && ((*ppszNext) > pchNextEOL)))
				(*ppszNext) = pchNextEOL;
			if ((*ppszNext) == NULL)
				/* this macro is terminated by the end of the input (oops!) */
				(*ppszNext) = strchr(pszInput, '\0');

			/* copy the macro into pszChunk and append the closing semi-colon */
			iLength = (*ppszNext) - pszStartChunk;
			strncpy(pszChunk, pszStartChunk, iLength);
			pszChunk[iLength] = '\0';
			strcat(pszChunk, ";");

			/* skip over the semi-colon if we did not insert it */
			if (*(*ppszNext) == ';')
				(*ppszNext) += 1;

			/* determine return value */
			if (((*ppszNext)[0] == ' ') || ((*ppszNext)[0] == '\n') || ((*ppszNext)[0] == '\0'))
				iRet = HTML_MACRO_END;
			else
				iRet = HTML_MACRO_MID;
			break;

		case '\0':
			/* we have reached the end of the input */
			iRet = HTML_END;
			break;

		default:
 	   		/* we are at the start of a word; find next possible chunk terminators */
			pchNextSpace = strchr(pszStartChunk, ' ');
			pchNextTag   = strchr(pszStartChunk, '<');
			pchNextMacro = strchr(pszStartChunk, '&');
			pchNextEOL   = strchr(pszStartChunk, '\n');

			/* decide which terminator came first */
			(*ppszNext) = pchNextSpace;
			if (((*ppszNext) == NULL) || ((pchNextTag != NULL) && ((*ppszNext) > pchNextTag)))
				(*ppszNext) = pchNextTag;
			if (((*ppszNext) == NULL) || ((pchNextMacro != NULL) && ((*ppszNext) > pchNextMacro)))
				(*ppszNext) = pchNextMacro;
			if (((*ppszNext) == NULL) || ((pchNextEOL != NULL) && ((*ppszNext) > pchNextEOL)))
				(*ppszNext) = pchNextEOL;
			if ((*ppszNext) == NULL)
				/* this chunk is terminated by the end of the input */
				(*ppszNext) = strchr(pszInput, '\0');

			/* copy the chunk into pszChunk */
			iLength = (*ppszNext) - pszStartChunk;
			strncpy(pszChunk, pszStartChunk, iLength);
			pszChunk[iLength] = '\0';

			/* determine return value */
			if (((*ppszNext)[0] == ' ') || ((*ppszNext)[0] == '\n') || ((*ppszNext)[0] == '\0'))
				iRet = HTML_WORD_END;
			else
				iRet = HTML_WORD_MID;
			break;
	}

	return (iRet);
}


int HTMLParseTag(char *pszTag)
/*
 * Decode an HTML tag.
 *
 * char *pszTag	- the tag to be decoded (including enclosing brackets)
 *
 * Return: HTML_UNKNOWN       - unrecognised tag
 *         HTML_PARAGRAPH     - new paragraph tag <P>
 *         HTML_ITALICS_START - opening italices tag <I>
 *         HTML_ITALICS_END   - closing italices tag </I>
 *         HTML_BOLD_START    - opening "bold" tag <B>, <EM> or <STRONG>
 *         HTML_BOLD_END      - closing "bold" tag </B>, </EM> or </STRONG>
 *         HTML_LINEBREAK     - line break tag <BR>
 */
{
	int iRet;

	if (stricmp(pszTag, "<P>") == 0)
		iRet = HTML_PARAGRAPH;
	else if (stricmp(pszTag, "<I>") == 0)
		iRet = HTML_ITALICS_START;
	else if (stricmp(pszTag, "</I>") == 0)
		iRet = HTML_ITALICS_END;
	else if (stricmp(pszTag, "<B>") == 0)
		iRet = HTML_BOLD_START;
	else if (stricmp(pszTag, "</B>") == 0)
		iRet = HTML_BOLD_END;
	else if (stricmp(pszTag, "<EM>") == 0)
		iRet = HTML_BOLD_START;
	else if (stricmp(pszTag, "</EM>") == 0)
		iRet = HTML_BOLD_END;
	else if (stricmp(pszTag, "<STRONG>") == 0)
		iRet = HTML_BOLD_START;
	else if (stricmp(pszTag, "</STRONG>") == 0)
		iRet = HTML_BOLD_END;
	else if (stricmp(pszTag, "<BR>") == 0)
		iRet = HTML_LINEBREAK;
	else
		iRet = HTML_UNKNOWN;

	return (iRet);
}


int HTMLParseMacro(char *pszMacro)
/*
 * Decode an HTML macro.
 *
 * char *pszMacro	- the macro to be decoded (including enclosing ampersand/semi-colon)
 *
 * Return: the ISO Latin-1 code corresponding to the macro
 *         -1 if this macro is not recognised
 *
 */
{
	char szStripped[10];
	char *pchSemiColon;
	int iRet;

	/* perform some sanity checking */
	if ((pszMacro[0] != '&') || ((pchSemiColon = strchr(pszMacro, ';')) == NULL))
		return (-1);

	/* strip the ampersand and semi-colon */
	strncpy(szStripped, pszMacro + 1, pchSemiColon - pszMacro - 1);
	szStripped[pchSemiColon - pszMacro - 1] = '\0';

	iRet = -1;
	if (szStripped[0] == '#') {
		/* the macro gives the ISO Latin-1 code */
		iRet = atoi(szStripped + 1);
		if ((iRet == 0) || (iRet > 255))
			iRet = -1;
	} else if (strcmp(szStripped, "quot") == 0)
		iRet = 34;
	else if (strcmp(szStripped, "amp") == 0)
		iRet = 38;
	else if (strcmp(szStripped, "lt") == 0)
		iRet = 60;
	else if (strcmp(szStripped, "gt") == 0)
		iRet = 62;
       else if (strcmp(szStripped, "nbsp") == 0)
		iRet = 160;
	else if (strcmp(szStripped, "iexcl") == 0)
		iRet = 161;
	else if (strcmp(szStripped, "cent") == 0)
		iRet = 162;
	else if (strcmp(szStripped, "pound") == 0)
		iRet = 163;
	else if (strcmp(szStripped, "curren") == 0)
		iRet = 164;
	else if (strcmp(szStripped, "yen") == 0)
		iRet = 165;
	else if (strcmp(szStripped, "brvbar") == 0)
		iRet = 166;
	else if (strcmp(szStripped, "sect") == 0)
		iRet = 167;
	else if (strcmp(szStripped, "uml") == 0)
		iRet = 168;
	else if (strcmp(szStripped, "copy") == 0)
		iRet = 169;
	else if (strcmp(szStripped, "ordf") == 0)
		iRet = 170;
	else if (strcmp(szStripped, "laquo") == 0)
		iRet = 171;
	else if (strcmp(szStripped, "not") == 0)
		iRet = 172;
	else if (strcmp(szStripped, "shy") == 0)
		iRet = 173;
	else if (strcmp(szStripped, "reg") == 0)
		iRet = 174;
	else if (strcmp(szStripped, "macr") == 0)
		iRet = 175;
	else if (strcmp(szStripped, "deg") == 0)
		iRet = 176;
	else if (strcmp(szStripped, "plusmn") == 0)
		iRet = 177;
	else if (strcmp(szStripped, "sup2") == 0)
		iRet = 178;
	else if (strcmp(szStripped, "sup3") == 0)
		iRet = 179;
	else if (strcmp(szStripped, "acute") == 0)
		iRet = 180;
	else if (strcmp(szStripped, "micro") == 0)
		iRet = 181;
	else if (strcmp(szStripped, "para") == 0)
		iRet = 182;
	else if (strcmp(szStripped, "middot") == 0)
		iRet = 183;
	else if (strcmp(szStripped, "cedil") == 0)
		iRet = 184;
	else if (strcmp(szStripped, "sup1") == 0)
		iRet = 185;
	else if (strcmp(szStripped, "ordm") == 0)
		iRet = 186;
	else if (strcmp(szStripped, "raquo") == 0)
		iRet = 187;
	else if (strcmp(szStripped, "frac14") == 0)
		iRet = 188;
	else if (strcmp(szStripped, "frac12") == 0)
		iRet = 189;
	else if (strcmp(szStripped, "frac34") == 0)
		iRet = 190;
	else if (strcmp(szStripped, "iquest") == 0)
		iRet = 191;
	else if (strcmp(szStripped, "Agrave") == 0)
		iRet = 192;
	else if (strcmp(szStripped, "Aacute") == 0)
		iRet = 193;
	else if (strcmp(szStripped, "Acirc") == 0)
		iRet = 194;
	else if (strcmp(szStripped, "Atilde") == 0)
		iRet = 195;
	else if (strcmp(szStripped, "Auml") == 0)
		iRet = 196;
	else if (strcmp(szStripped, "Aring") == 0)
		iRet = 197;
	else if (strcmp(szStripped, "AElig") == 0)
		iRet = 198;
	else if (strcmp(szStripped, "Ccedil") == 0)
		iRet = 199;
	else if (strcmp(szStripped, "Egrave") == 0)
		iRet = 200;
	else if (strcmp(szStripped, "Eacute") == 0)
		iRet = 201;
	else if (strcmp(szStripped, "Ecirc") == 0)
		iRet = 202;
	else if (strcmp(szStripped, "Euml") == 0)
		iRet = 203;
	else if (strcmp(szStripped, "Igrave") == 0)
		iRet = 204;
	else if (strcmp(szStripped, "Iacute") == 0)
		iRet = 205;
	else if (strcmp(szStripped, "Ihat") == 0)
		iRet = 206;
	else if (strcmp(szStripped, "Iuml") == 0)
		iRet = 207;
	else if (strcmp(szStripped, "ETH") == 0)
		iRet = 208;
	else if (strcmp(szStripped, "Ntilde") == 0)
		iRet = 209;
	else if (strcmp(szStripped, "Ograve") == 0)
		iRet = 210;
	else if (strcmp(szStripped, "Oacute") == 0)
		iRet = 211;
	else if (strcmp(szStripped, "Ocirc") == 0)
		iRet = 212;
	else if (strcmp(szStripped, "Otilde") == 0)
		iRet = 213;
	else if (strcmp(szStripped, "Ouml") == 0)
		iRet = 214;
	else if (strcmp(szStripped, "times") == 0)
		iRet = 215;
	else if (strcmp(szStripped, "Oslash") == 0)
		iRet = 216;
	else if (strcmp(szStripped, "Ugrave") == 0)
		iRet = 217;
	else if (strcmp(szStripped, "Uacute") == 0)
		iRet = 218;
	else if (strcmp(szStripped, "Uhat") == 0)
		iRet = 219;
	else if (strcmp(szStripped, "Uuml") == 0)
		iRet = 220;
	else if (strcmp(szStripped, "Yacute") == 0)
		iRet = 221;
	else if (strcmp(szStripped, "THORN") == 0)
		iRet = 222;
	else if (strcmp(szStripped, "szlig") == 0)
		iRet = 223;
	else if (strcmp(szStripped, "agrave") == 0)
		iRet = 224;
	else if (strcmp(szStripped, "aacute") == 0)
		iRet = 225;
	else if (strcmp(szStripped, "acirc") == 0)
		iRet = 226;
	else if (strcmp(szStripped, "atilde") == 0)
		iRet = 227;
	else if (strcmp(szStripped, "auml") == 0)
		iRet = 228;
	else if (strcmp(szStripped, "aring") == 0)
		iRet = 229;
	else if (strcmp(szStripped, "aelig") == 0)
		iRet = 230;
	else if (strcmp(szStripped, "ccedil") == 0)
		iRet = 231;
	else if (strcmp(szStripped, "egrave") == 0)
		iRet = 232;
	else if (strcmp(szStripped, "eacute") == 0)
		iRet = 233;
	else if (strcmp(szStripped, "ehat") == 0)
		iRet = 234;
	else if (strcmp(szStripped, "euml") == 0)
		iRet = 235;
	else if (strcmp(szStripped, "igrave") == 0)
		iRet = 236;
	else if (strcmp(szStripped, "iacute") == 0)
		iRet = 237;
	else if (strcmp(szStripped, "ihat") == 0)
		iRet = 238;
	else if (strcmp(szStripped, "iuml") == 0)
		iRet = 239;
	else if (strcmp(szStripped, "eth") == 0)
		iRet = 240;
	else if (strcmp(szStripped, "ntilde") == 0)
		iRet = 241;
	else if (strcmp(szStripped, "ograve") == 0)
		iRet = 242;
	else if (strcmp(szStripped, "oacute") == 0)
		iRet = 243;
	else if (strcmp(szStripped, "ohat") == 0)
		iRet = 244;
	else if (strcmp(szStripped, "otilde") == 0)
		iRet = 245;
	else if (strcmp(szStripped, "ouml") == 0)
		iRet = 246;
	else if (strcmp(szStripped, "ugrave") == 0)
		iRet = 249;
	else if (strcmp(szStripped, "uacute") == 0)
		iRet = 250;
	else if (strcmp(szStripped, "uhat") == 0)
		iRet = 251;
	else if (strcmp(szStripped, "uuml") == 0)
		iRet = 252;
	else if (strcmp(szStripped, "yacute") == 0)
		iRet = 253;
	else if (strcmp(szStripped, "thorn") == 0)
		iRet = 254;
	else if (strcmp(szStripped, "yuml") == 0)
		iRet = 255;
	else
		iRet = -1;

	return (iRet);
}
