/*
 * qthreads.c
 *
 * Threads for use with quote databases.
 *
 *       Created: 27th January, 1997
 * Last modified: 10th April, 1997
 *
 * (C) 1997 Nicholas Paul Sheppard
 *
 * This file is distributed under the GNU General Public License. See the
 * file COPYING for details.
 */

#include <stdlib.h>
#include <time.h>
#include "quotes.h"
#include "threads.h"


void ThreadFindAllQuotes(THREADINFO *pti)
/*
 * Find all quotes in a database. The code of each quote is returned to the
 * calling window by sending a QM_FOUND message to it with the first parameter
 * set to be a pointer to the code. The thread sends a QM_DONE message to the
 * caller when it is finished.
 */
{
	HAB		hab;
	HMQ		hmq;
	QUOTESEARCH	qs;

	/* set up a message queue */
	if ((hab = WinInitialize(0)) == NULLHANDLE)
		return;
	hmq = WinCreateMsgQueue(hab, 0);

	/* perform the search */
	if (QuoteFullSearchInit(&(pti->pps->qdb), &qs)) {
		WinSendMsg(pti->hwndCaller, QM_FOUND, MPFROMP(qs.szLastCode), 0);
		while (QuoteFullSearchNext(&qs))
			WinSendMsg(pti->hwndCaller, QM_FOUND, MPFROMP(qs.szLastCode), 0);
	}
	WinSendMsg(pti->hwndCaller, QM_DONE, 0, 0);

	/* clean up */
	WinDestroyMsgQueue(hmq);
	WinTerminate(hab);
	_endthread();
}


void ThreadFindKeywords(THREADINFO *pti)
/*
 * Find all quotes in a database with the keyword given in the pData field of
 * the THREADINFO structure. The code of each quote is returned to the calling
 * window by sending a QM_FOUND message to it with the first parameter set to
 * be a pointer to the code. The thread sends a QM_DONE message to the caller
 * when it is finished.
 */
{
	HAB		hab;
	HMQ		hmq;
	QUOTESEARCH	qs;

	/* set up a message queue */
	if ((hab = WinInitialize(0)) == NULLHANDLE)
		return;
	hmq = WinCreateMsgQueue(hab, 0);

	/* perform the search */
	if (QuoteKeySearchInit(&(pti->pps->qdb), (char *)(pti->pData), &qs)) {
		WinSendMsg(pti->hwndCaller, QM_FOUND, MPFROMP(qs.szLastCode), 0);
		while (QuoteKeySearchNext(&qs))
			WinSendMsg(pti->hwndCaller, QM_FOUND, MPFROMP(qs.szLastCode), 0);
	}
	WinSendMsg(pti->hwndCaller, QM_DONE, 0, 0);

	/* clean up */
	WinDestroyMsgQueue(hmq);
	WinTerminate(hab);
	_endthread();
}


void ThreadFindQuoteText(THREADINFO *pti)
/*
 * Find all quotes in a database containing the text given in the pData field
 * of the THREADINFO structure. The code of each quote is returned to the
 * calling window by sending a QM_FOUND message to it with the first parameter
 * set to be a pointer to the code. The thread sends a QM_DONE message to the
 * caller when it is finished.
 */
{
	HAB		hab;
	HMQ		hmq;
	QUOTESEARCH	qs;

	/* set up a message queue */
	if ((hab = WinInitialize(0)) == NULLHANDLE)
		return;
	hmq = WinCreateMsgQueue(hab, 0);

	/* perform the search */
	if (QuoteTextSearchInit(&(pti->pps->qdb), (char *)(pti->pData), &qs)) {
		WinSendMsg(pti->hwndCaller, QM_FOUND, MPFROMP(qs.szLastCode), 0);
		while (QuoteTextSearchNext(&qs))
			WinSendMsg(pti->hwndCaller, QM_FOUND, MPFROMP(qs.szLastCode), 0);
	}
	WinSendMsg(pti->hwndCaller, QM_DONE, 0, 0);

	/* clean up */
	WinDestroyMsgQueue(hmq);
	WinTerminate(hab);
	_endthread();
}


void ThreadFindAuthorsQuotes(THREADINFO *pti)
/*
 * Find all quotes in a database from the author whose code is given in the
 * pData field of the the THREADINFO structure. The code of each quote is
 * returned to the calling window by sending a QM_FOUND message to it with the
 * first parameter set to be a pointer to the code. The thread sends a QM_DONE
 * message to the caller when it is finished.
 */
{
	HAB		hab;
	HMQ		hmq;
	QUOTESEARCH	qs;

	/* set up a message queue */
	if ((hab = WinInitialize(0)) == NULLHANDLE)
		return;
	hmq = WinCreateMsgQueue(hab, 0);

	/* perform the search */
	if (QuoteAuthorSearchInit(&(pti->pps->qdb), (char *)(pti->pData), &qs)) {
		WinSendMsg(pti->hwndCaller, QM_FOUND, MPFROMP(qs.szLastCode), 0);
		while (QuoteAuthorSearchNext(&qs))
			WinSendMsg(pti->hwndCaller, QM_FOUND, MPFROMP(qs.szLastCode), 0);
	}
	WinSendMsg(pti->hwndCaller, QM_DONE, 0, 0);

	/* clean up */
	WinDestroyMsgQueue(hmq);
	WinTerminate(hab);
	_endthread();
}


void ThreadFindRandomQuote(THREADINFO *pti)
/*
 * Choose a random quote from a database. When one has been chosen, a QM_FOUND
 * message is sent to the calling window with the first parameter set to be a
 * pointer to the code chosen. If the database is empty, place NULL in this
 * first parameter instead.
 */
{
	HAB		hab;
	HMQ		hmq;
	QUOTESEARCH	qs;
	int		i, iQuote;

	/* set up a message queue */
	if ((hab = WinInitialize(0)) == NULLHANDLE)
		return;
	hmq = WinCreateMsgQueue(hab, 0);

	/* count the number of quotes in the database */
	i = 0;
	if (QuoteFullSearchInit(&(pti->pps->qdb), &qs))
		while (QuoteFullSearchNext(&qs))
			i++;

	if (i == 0)
		/* empty database */
		WinSendMsg(pti->hwndCaller, QM_FOUND, MPFROMP(NULL), 0);
	else {
		/* choose a random quote */
		if (i > 1) {
			srand(time(NULL));
			iQuote = (rand() % i) + 1;
		} else
			iQuote = 1;
		for (QuoteFullSearchInit(&(pti->pps->qdb), &qs), i = 0; i < iQuote; QuoteFullSearchNext(&qs), i++);
		WinSendMsg(pti->hwndCaller, QM_FOUND, MPFROMP(qs.szLastCode), 0);
	}

	/* clean up */
	WinDestroyMsgQueue(hmq);
	WinTerminate(hab);
	_endthread();
}
